@@ -27,7 +27,8 @@ from collections import defaultdict
 multiFLEXXDetectors = 31*5
 reFloat = r'-?\d*\.\d*'
 factorsqrtEK = 0.694692
-supportedRawFormats = ['hdf','MultiFLEXX','FlatCone']
+supportedRawFormats = ['hdf','','dat']
+supportedInstruments = ['CAMEA','MultiFLEXX','FlatCone','Bambus']
 supportedConvertedFormats = ['nxs']
 
 def cosd(x):
@@ -74,7 +75,9 @@ HDFTranslation = {'sample':'/entry/sample',
                   'scanCommand':'entry/scancommand',
                   'title':'entry/title',
                   'absoluteTime':'entry/control/absolute_time',
-                  'protonBeam':'entry/proton_beam/data'
+                  'protonBeam':'entry/proton_beam/data',
+                  'singleDetector1':'entry/CAMEA/segment_1/data',
+                  'singleDetector8':'entry/CAMEA/segment_8/data'
 }
 ## Default dictionary to perform on loaded data, i.e. take the zeroth element, swap axes, etc
 
@@ -115,6 +118,16 @@ extraAttributes = ['name','fileLocation','twoTheta']
 possibleAttributes = list(HDFTranslation.keys())+list(HDFInstrumentTranslation.keys())+extraAttributes
 possibleAttributes.sort(key=lambda v: v.lower())
 
+analyzerLimits = {'CAMEA':7,
+                  'Bambus': 4,
+                  'MultiFLEXX': 4,
+                  'FlatCone':1}
+
+detectorLimits = {'CAMEA':103,
+                  'Bambus':19,
+                  'MultiFLEXX':31,
+                  'FlatCone':32}
+
 class DataFile(object):
     """Object to load and keep track of HdF files and their conversions"""
     def __init__(self,fileLocation=None):
@@ -131,6 +144,8 @@ class DataFile(object):
 	            self.type='hdf'
             elif os.path.splitext(fileLocation)[1]=='': # No extension
                 self.type = 'MultiFLEXX'
+            elif fileLocation.split('.')[-1]=='dat':
+                self.type = 'Bambus'
             else:
                 raise AttributeError('File is not of type nxs or hdf.')
             self.name = os.path.basename(fileLocation)
@@ -139,7 +154,7 @@ class DataFile(object):
             self._mask = False
             self.absolutNormalized = False
 
-            if not self.type == 'MultiFLEXX':
+            if self.type in ['hdf','nxs']:
                 with hdf.File(fileLocation,mode='r') as f:
                     self.sample = MJOLNIR.Data.Sample.Sample(sample=f.get(HDFTranslation['sample']))
                     instr = getInstrument(f)
@@ -183,6 +198,8 @@ class DataFile(object):
                     self.userEmail = np.array(f.get(HDFTranslation['email']))[0]
                     self.userAddress = np.array(f.get(HDFTranslation['address']))[0]
                     self.userAffiliation = np.array(f.get(HDFTranslation['affiliation']))[0]
+                    self.singleDetector1 = np.array(f.get(HDFTranslation['singleDetector1']))
+                    self.singleDetector8 = np.array(f.get(HDFTranslation['singleDetector8']))
 
                     # Monochromator
 
@@ -284,9 +301,10 @@ class DataFile(object):
                     self.mask = np.zeros_like(self.I,dtype=bool)
                     if self.binning == 8:
                         self.mask[:,:,:2] = True
-            else: # type is multiFLEXX
+            elif self.type == 'MultiFLEXX': # type is multiFLEXX
                 self.loadMultiFLEXXData(fileLocation)
-            
+            elif self.type == 'Bambus':
+                self.loadBambusData(fileLocation)
             self.scanSteps = self.scanValues.shape[1]
             if True:
                 for attr in dir(self):
@@ -310,6 +328,14 @@ class DataFile(object):
                 else:
                     setattr(self,key,None)
         else: # Create an empty data set
+            self.instrument = 'CAMEA'
+            self.type = 'Unknown'
+
+        if self.instrument == 'CAMEA':
+            self.EPrDetector = 8 
+        elif self.type in ['MultiFLEXX','FlatCone','Bambus']:
+            self.EPrDetector = 1
+        else:
             pass
         
 
@@ -382,6 +408,20 @@ class DataFile(object):
             else:
                 self._A4 = A4
 
+    @property
+    def dasel(self):
+        if hasattr(self,'detectorSelection') and hasattr(self,'analyzerSelection'):
+            return self.detectorSelection,self.analyzerSelection
+        else:
+            return None,None
+
+    @dasel.getter
+    def dasel(self):
+        if hasattr(self,'detectorSelection') and hasattr(self,'analyzerSelection'):
+            return self.detectorSelection,self.analyzerSelection
+        else:
+            return None,None
+
     @property
     def binning(self):
         return self._binning
@@ -688,6 +728,7 @@ class DataFile(object):
             self.multiData = np.array([np.array(x.split()[1:],dtype=int) for x in dataPart[2::3]])
             self.multiData = self.multiData[:,:155] # Remove additional 5 zeros in data file
             self.type = 'MultiFLEXX'
+            self.instrument = 'MultiFLEXX'
             if 'Finished' in dataPart[-1]:
                 self.endTime = dataPart[-1].replace('Finished at ','')
             else:
@@ -700,9 +741,11 @@ class DataFile(object):
                 self.pntData = np.array([np.array(x.split(),dtype=float) for x in dataPart[1:split]])
                 self.multiData = np.array([np.array(x.split(),dtype=int) for x in dataPart[split+1:]])
                 self.type = 'FlatCone'
+                self.instrument = 'FlatCone'
             else:
                 self.pntData = np.array([np.array(x.split(),dtype=float) for x in dataPart[1:] if x[:8]!='Finished'])
                 self.type='1D'
+                self.instrument = 'UNKNOWN'
                 
 
         ## Extract the data from above
@@ -805,15 +848,292 @@ class DataFile(object):
             
         self.twotheta = self.A4
 
+
+    def loadBambusData(self,fileLocation,calibrationFile=None):
+        self.fileLocation = fileLocation
+        self.possibleBinnings = [1] # Standard value (1 energy/detector)
+        self.binning = 1
+        self.instrument = 'Bambus'
+        self.type = self.instrument
         
+
+        ## No dasel
+        self.analyzerSelection = 0
+        self.detectorSelection = 0
+
+        with open(fileLocation) as f:
+            dataString = f.readlines()
+
+
+
+        # Format for starting time is 2021-11-26 10:45:05
+        searchString = r'([0-9]+)-([0-9]+)-([0-9]+)\s([0-9]+):([0-9]+):([0-9]+)'
+        
+        matches = re.findall(searchString,dataString[0])
+        if len(matches) == 0:
+            self.startTime = 'UNKNONW'
+        else:
+            self.startTime = '-'.join(matches[0][:3])+' '+':'.join(matches[0][3:])
+        self.endTime = 'UNKNOWN'
+
+        if calibrationFile is None:
+            calibrationFile = MJOLNIR.__bambusNormalization__
+
+        detectors = 20
+        self.mask = False
+
+        calibrationData = np.genfromtxt(calibrationFile,skip_header=1,delimiter=',')
+        amplitude = calibrationData[:,3]
+        background = calibrationData[:,6] 
+        bound = calibrationData[:,7:9]
+        final_energy = calibrationData[:,4]
+        width = calibrationData[:,5]
+        A4 = -calibrationData[:,9]
+
+        EfTable = np.array([amplitude,final_energy,width,background]).T
+        calibrations=[[EfTable,A4,bound]]
+        bound =  np.array(detectors*[0,1],dtype=int).reshape(-1,2)
+
+        self.instrumentCalibrationEdges = bound
+        self.instrumentCalibrationEf = EfTable
+        self.instrumentCalibrationA4 = A4
+
+        self.instrumentCalibrations = calibrations
+
+        ## Load header data:
+
+        ## Format is "#    param : value"
+
+        parameters = {}
+
+        for I,line in enumerate(dataString):
+            if line[:2] == '# ':
+                param,*value = [x.strip() for x in line.split(':')]
+                param = param.replace('#','').strip()
+                if len(value)>1:
+                    value = ':'.join(value)
+                parameters[param] = value[0]
+            else:
+                if line.find('### Scan data') >-1:
+                    dataline = I
+                    break
+
+        # Convert from ch number in data file to detector,energy (A1,A2,...,20D,20E)
+        detectorMap = np.array([ 0,  1,  2, 27,  3, 50, 51, 52, 77, 53,  4,  5,  6, 28,  7, 54, 55,
+            56, 78, 57,  8,  9, 10, 29, 11, 58, 59, 60, 79, 61, 12, 13, 14, 30,
+            15, 62, 63, 64, 80, 65, 16, 17, 18, 31, 19, 66, 67, 68, 81, 69, 20,
+            21, 22, 32, 23, 70, 71, 72, 82, 73, 24, 25, 26, 33, 37, 74, 75, 76,
+            83, 87, 38, 39, 40, 34, 41, 88, 89, 90, 84, 91, 42, 43, 44, 35, 45,
+            92, 93, 94, 85, 95, 46, 47, 48, 36, 49, 96, 97, 98, 86, 99])
+
+
+        # Load data
+        titles = dataString[dataline+1].strip().split('\t')
+        units =  dataString[dataline+2].strip().split('\t')
+
+
+        # remove the '# ' part of the first title and unit
+        titles[0] = titles[0].replace('#','').strip()
+        units[0] = units[0].replace('#','').strip()
+
+
+        # Find splitter in data set ';'
+        
+        splitter = titles.index(';')
+        
+
+        # Shape becomes [scan points, timer + mon1 + mon2 + chsum + 100 detectors + events]
+        data = [[float(x) for x in line.split('\t')[splitter+1:]] for line in dataString[dataline+3:-1]]
+        scanData = [[float(x) for x in line.split('\t')[:splitter]] for line in dataString[dataline+3:-1]]
+        
+        if len(data[-1]) == 0: # empty last line
+            data = np.array(data[:-1])
+            scanData = np.array(scanData[:-1])
+        else:
+            data = np.array(data)
+            scanData = np.array(scanData)
+            
+
+        #reshape into [scan points, 20 wedges, 5 energies]
+        self.I = np.array([d[detectorMap] for d in data[:,4:-1]]).reshape(-1,20*5,1)
+
+
+        self.timer = data[:,0].astype(float)
+        self.mon1,self.mon2,self.ctot,self.events = data[:,[1,2,3,-1]].astype(int).T
+
+        self.scanParameters = np.asarray(titles[:splitter])
+        self.scanUnits = np.asarray(units[:splitter])
+        self.scanValues = np.asarray(scanData[:,:splitter]).T.astype(float)
+
+        self.scanParameters[0] = self.scanParameters[0].replace('#','').strip()
+        self.scanUnits[0] = self.scanUnits[0].replace('#','').strip()
+
+        ### Move last scan parameter to first position in lists
+
+        self.scanParameters = self.scanParameters[[-1,*range(len(self.scanParameters)-1)]]
+        self.scanUnits = self.scanUnits[[-1,*range(len(self.scanParameters)-1)]]
+        self.scanValues = self.scanValues[[-1,*range(len(self.scanParameters)-1)]]
+
+
+        self.__dict__.update(parameters)
+
+
+
+
+        #### Import sample correctly
+
+        sample = {}
+        def extractFromParenthesis(string):
+            return list(map(float,re.findall(r'(?<=\().*?(?=\))',string)[0].split(',')))
+
+
+        cell = np.concatenate([extractFromParenthesis(dat) for dat in [self.Sample_lattice,self.Sample_angles]])
+
+        for param,value in zip(['a','b','c','alpha','beta','gamma'],cell):
+            sample[param] = value
+
+
+        q1 = np.array(extractFromParenthesis(self.Sample_orient1))
+        q2 = np.array(extractFromParenthesis(self.Sample_orient2))
+
+
+
+        Ei = 10.0 # TODO: What is the good solution here? Dummy incoming energy needed to calcualte UB
+        k = np.sqrt(Ei)*factorsqrtEK
+
+        Cell = TasUBlib.calcCell(cell)
+        B = TasUBlib.calculateBMatrix(Cell)
+
+        A3offset = float(self.Sample_psi0.split(' ')[0])
+
+        A41 = TasUBlib.calTwoTheta(B,[*q1,Ei,Ei],-1)
+        A31 = TasUBlib.calcTheta(k,k,A41)+A3offset
+        A42 = TasUBlib.calTwoTheta(B,[*q2,Ei,Ei],-1)
+        A32 = TasUBlib.calcTheta(k,k,A42)
+
+        planeVector1 = list(q1)
+        planeVector1.append(A31) # A3 
+        planeVector1.append(A41) # A4
+        [planeVector1.append(0.0) for _ in range(2)]# append values for gonios set to zero
+        planeVector1.append(Ei)
+        planeVector1.append(Ei)
+
+        planeVector2 = list(q2)
+        planeVector2.append(A32) # A3 
+        planeVector2.append(A42) # A4 
+        [planeVector2.append(0.0) for _ in range(2)]# append values for gonios set to zero
+        planeVector2.append(Ei)
+        planeVector2.append(Ei)
+
+        # add correct angle in theta between the two reflections
+        between = TasUBlib.tasAngleBetweenReflections(B,np.array(planeVector1),np.array(planeVector2))
+
+        planeVector2[3]+=between
+
+        sample['projectionVector1']=np.array(planeVector1)
+        sample['projectionVector2']=np.array(planeVector2)
+
+        sample['name'] = self.Sample_samplename
+
+
+        self.sample = MJOLNIR.Data.Sample.Sample(**sample)
+
+        for sP,sV in zip(self.scanParameters,self.scanValues):
+            setattr(self,sP,sV)
+        
+
+        monoQ = np.array(float(self.mono_value.split(' ')[0]))
+        self.Ei = np.array(float(self.Ei_value.split(' ')[0]))
+        self.A4 = np.array([float(self.stt_value.split(' ')[0])])
+
+        if isinstance(self.sth_value,str):
+            self._A3 = np.array(float(self.sth_value.split(' ')[0]))
+        else:
+            self._A3 = self.sth_value
+
+        self.A4Off = 0.0
+        self.A3Off = None
+
+        self.countingTime = np.array([self.etime[0],*np.diff(self.etime)])
+        
+
+        nameSwaps = [['filename','name'],
+                    ['info','scanCommand'],
+                    ['Exp_remark','comment'],
+                    
+                    #['MAG','magneticField'],
+                    ['mon1','Monitor'],
+                    ['timer','Time'],
+                    #['TT','temperature'],
+                    #['comnd','scanCommand'],
+                    #['instr','instrument'],
+                    #['EI','Ei'],
+                    
+                    ['Exp_localcontact','localContact'],
+                    ['Exp_proposal','proposalID'],
+                    ['Exp_title','title'],
+                    ['Exp_users','userName'],
+
+                    
+                    
+                    
+                    #['DA','analyzerDSpacing'],
+                    #['expno','experimentIdentifier'],
+                    ['localContact','localContactName'],
+                    #['date','startTime']
+                    ]
+
+
+        if not hasattr(self,'electricField'):
+            self.electricField = None
+        if not hasattr(self,'magneticField'):
+            self.magneticField = None
+
+        if not hasattr(self,'temperature'):
+            self.temperature = None
+            
+        self.twotheta = self.A4
+
+
+        ## Format scanCommand
+
+        def updateKeyName(obj,key,newName):
+            if not hasattr(obj,key):
+                return
+            setattr(obj,newName,getattr(obj,key))
+            delattr(obj,key)
+
+
+        for pair in nameSwaps:
+            updateKeyName(self,pair[0],pair[1])
+
+    @_tools.KwargChecker()
+    def calcualteDataIndexFromDasel(self,detectorSelection=None,analyzerSelection=None):
+        if detectorSelection is None:
+            detectorSelection = self.detectorSelection
+        if analyzerSelection is None:
+            analyzerSelection = self.analyzerSelection
+
+        if self.instrument == 'CAMEA':
+            if detectorSelection > detectorLimits['CAMEA']:
+                raise AttributeError('Provided detectorSelection is out of range. Recieved {} and instrument has {} detectors enumerated [0-{}]'.format(detectorSelection,detectorLimits['CAMEA']+1,detectorLimits['CAMEA']))
+            if analyzerSelection > analyzerLimits['CAMEA']:
+                raise AttributeError('Provided analyzerSelection is out of range. Recieved {} and instrument has {} analyzers enumerated [0-{}]'.format(analyzerSelection,analyzerLimits['CAMEA']+1,analyzerLimits['CAMEA']))
+            analyzerPixels = self.instrumentCalibrationEdges[analyzerSelection+detectorSelection*8] 
+            return detectorSelection,range(int(analyzerPixels[0]),int(analyzerPixels[1]))
+        
+        else:
+            if detectorSelection > detectorLimits[self.instrument]:
+                raise AttributeError('Provided detectorSelection is out of range. Recieved {} and instrument has {} detectors enumerated [0-{}]'.format(detectorSelection,detectorLimits[self.instrument]+1,detectorLimits[self.instrument]))
+            if analyzerSelection > analyzerLimits['Bambus']:
+                raise AttributeError('Provided analyzerSelection is out of range. Recieved {} and instrument has {} analyzers enumerated [0-{}]'.format(analyzerSelection,analyzerLimits[self.instrument]+1,analyzerLimits[self.instrument]))
+            # Calcualte detector number from detector and analyzer. As there are 5 energies(analyzers) per wedge
+            return detectorSelection*(analyzerLimits[self.instrument]+1)+analyzerSelection,[0] # Last index is to be able to sum over
+    
+    
     @_tools.KwargChecker()
     def convert(self,binning=None,printFunction=None):
-        if self.instrument == 'CAMEA':
-            EPrDetector = 8 
-            if binning is None:
-                binning = self.binning
-        elif self.type in ['MultiFLEXX','FlatCone']:
-            EPrDetector = 1
+        if self.instrument == 'CAMEA' or self.type in ['MultiFLEXX','FlatCone','Bambus']:
             if binning is None:
                 binning = self.binning
         else:
@@ -826,13 +1146,14 @@ class DataFile(object):
         EfNormalization = self.instrumentCalibrationEf.copy()
         A4Normalization = self.instrumentCalibrationA4.copy()#np.array(instrument.get('calib{}/a4offset'.format(str(binning))))
         EdgesNormalization = self.instrumentCalibrationEdges.copy()#np.array(instrument.get('calib{}/boundaries'.format(str(binning))))
+        self.counts = self.I.copy()
         Data = self.I.copy()#np.array(instrument.get('detector/data'))
 
 
         detectors = Data.shape[1]
         steps = Data.shape[0]
         
-        if self.type in ['MultiFLEXX','FlatCone']:
+        if self.type in ['MultiFLEXX','FlatCone','Bambus']:
             Data.shape = (Data.shape[0],Data.shape[1],-1)
 
         A4Zero = self.A4Off#file.get('entry/sample/polar_angle_zero')
@@ -850,27 +1171,27 @@ class DataFile(object):
             A3Zero = np.deg2rad(np.array(A3Zero))
 
         A4 = np.deg2rad(A4Normalization)
-        A4=A4.reshape(detectors,binning*EPrDetector,order='C')
+        A4=A4.reshape(detectors,binning*self.EPrDetector,order='C')
 
-        PixelEdge = EdgesNormalization.reshape(detectors,EPrDetector,binning,2).astype(int)
+        PixelEdge = EdgesNormalization.reshape(detectors,self.EPrDetector,binning,2).astype(int)
         A4File = self.A4.copy()
         
         A4File = A4File.reshape((-1,1,1))
 
-        A4Mean = (A4.reshape((1,detectors,binning*EPrDetector))+np.deg2rad(A4File-A4Zero))
+        A4Mean = (A4.reshape((1,detectors,binning*self.EPrDetector))+np.deg2rad(A4File-A4Zero))
         
-        Intensity=np.zeros((Data.shape[0],Data.shape[1],EPrDetector*binning),dtype=int)
+        Intensity=np.zeros((Data.shape[0],Data.shape[1],self.EPrDetector*binning),dtype=int)
         for i in range(detectors): # for each detector
-            for j in range(EPrDetector):
+            for j in range(self.EPrDetector):
                 for k in range(binning):
                     Intensity[:,i,j*binning+k] = np.sum(Data[:,i,PixelEdge[i,j,k,0]:PixelEdge[i,j,k,1]],axis=1)
 
         
-        EfMean = EfNormalization[:,1].reshape(1,A4.shape[0],EPrDetector*binning)
+        EfMean = EfNormalization[:,1].reshape(1,A4.shape[0],self.EPrDetector*binning)
         EfNormalization = EfNormalization[:,0]#.reshape(1,A4.shape[0],EPrDetector*binning)#
         #EfNormalization = EfNormalization[:,0]*(np.sqrt(2*np.pi)*EfNormalization[:,2])
         
-        EfNormalization.shape = (1,A4.shape[0],EPrDetector*binning)
+        EfNormalization.shape = (1,A4.shape[0],self.EPrDetector*binning)
         A3 = np.deg2rad(np.array(self.A3).copy())+A3Zero #file.get('/entry/sample/rotation_angle/')
         if A3.shape[0]==1:
             A3 = A3*np.ones((steps))
@@ -899,7 +1220,7 @@ class DataFile(object):
         if DeltaE.shape[0]==1:
             DeltaE = DeltaE*np.ones((steps,1,1))
         Monitor = self.Monitor.copy().reshape((steps,1,1))
-        Monitor = Monitor*np.ones((1,detectors,EPrDetector*binning))
+        Monitor = Monitor*np.ones((1,detectors,self.EPrDetector*binning))
         Normalization = EfNormalization*np.ones((steps,1,1))
 
        
@@ -1468,8 +1789,9 @@ class DataFile(object):
             values = ['analyzer'+x.replace('_',' ').title().replace(' ','') for x in attributes]
             units = ['anstrom','mev','degree','degree']
             for att,value,unit in zip(attributes,values,units):
-                dset = ana.create_dataset(att,(1,),'float32')
-                dset[0] = getattr(self,value)
+                data = getattr(self,value)
+                dset = ana.create_dataset(att,(len(data),),'float32')
+                dset[:len(data)] = data
                 if not unit is None:
                     dset.attrs['units'] = np.string_(unit)
                 
@@ -1852,39 +2174,61 @@ def shallowRead(files,parameters):
     
     for file in files:
         vals = []
-        with hdf.File(file,mode='r') as f:
-            instr = getInstrument(f)
+        if os.path.splitext(file)[1] == 'hdf': # if an hdf file
+            with hdf.File(file,mode='r') as f:
+                instr = getInstrument(f)
+                for p in parameters:
+                    if p == 'name':
+                        v = os.path.basename(file)
+                        vals.append(v)
+                        continue
+                    elif p == 'fileLocation':
+                        v = os.path.dirname(file)
+                        vals.append(v)
+                        continue
+                    elif p == 'twoTheta':
+                        A4 = np.array(instr.get(HDFInstrumentTranslation['A4']))
+                        for func,args in HDFInstrumentTranslationFunctions['A4']:
+                            A4 = getattr(A4,func)(*args)
+                        A4Offset = np.array(instr.get(HDFInstrumentTranslation['A4Offset']))
+                        for func,args in HDFInstrumentTranslationFunctions['A4Offset']:
+                            A4Offset = getattr(A4Offset,func)(*args)
+                        vals.append(A4-A4Offset)
+                        continue
+                    elif p in HDFTranslation:
+                        v = np.array(f.get(HDFTranslation[p]))
+                        TrF= HDFTranslationFunctions
+                    elif p in HDFInstrumentTranslation:
+                        v = np.array(instr.get(HDFInstrumentTranslation[p]))
+                        TrF= HDFInstrumentTranslationFunctions
+                    else:
+                        raise AttributeError('Parameter "{}" not found'.format(p))
+                    for func,args in TrF[p]:
+                        v = getattr(v,func)(*args)
+                    
+                    vals.append(v)
+                values.append(vals)
+        else:
+            df = DataFile(file)
             for p in parameters:
-                if p == 'name':
-                    v = os.path.basename(file)
-                    vals.append(v)
-                    continue
-                elif p == 'fileLocation':
-                    v = os.path.dirname(file)
-                    vals.append(v)
-                    continue
-                elif p == 'twoTheta':
-                    A4 = np.array(instr.get(HDFInstrumentTranslation['A4']))
-                    for func,args in HDFInstrumentTranslationFunctions['A4']:
-                        A4 = getattr(A4,func)(*args)
-                    A4Offset = np.array(instr.get(HDFInstrumentTranslation['A4Offset']))
-                    for func,args in HDFInstrumentTranslationFunctions['A4Offset']:
-                        A4Offset = getattr(A4Offset,func)(*args)
-                    vals.append(A4-A4Offset)
-                    continue
-                elif p in HDFTranslation:
-                    v = np.array(f.get(HDFTranslation[p]))
-                    TrF= HDFTranslationFunctions
-                elif p in HDFInstrumentTranslation:
-                    v = np.array(instr.get(HDFInstrumentTranslation[p]))
-                    TrF= HDFInstrumentTranslationFunctions
-                else:
-                    raise AttributeError('Parameter "{}" not found'.format(p))
-                for func,args in TrF[p]:
-                    v = getattr(v,func)(*args)
                 
-                vals.append(v)
+                try:
+                    vals.append(getattr(df,p))
+                except:
+                    if 'sample' in p:
+                        try:
+                            vals.append(getattr(df.sample,p.replace('sample','').lower()))
+                        except:
+                            vals.append('Not Found :S')
+                    else:
+                        try:
+                            vals.append(getattr(df,p.lower()))
+                        except:
+                            vals.append('Not Found :S')
             values.append(vals)
+            
+
+
     return values    
 
 
