@@ -1,5 +1,6 @@
 import os
 import sys
+from typing import Type
 
 sys.path.append('.')
 sys.path.append('..')
@@ -13,6 +14,9 @@ import matplotlib.pyplot as plt
 import numpy as np
 from matplotlib.widgets import Slider
 from MJOLNIR import _tools
+from MJOLNIR._interactiveSettings import Viewer3DSettings, States, cut1DHolder
+from MJOLNIR.Data.DraggableShapes import clearBoxes, DraggableCircle,DraggableRectangle, prepareInteractiveCuttingView3D,\
+    extractCut1DPropertiesRectangle, extractCut1DPropertiesCircle
 import functools
 
 pythonVersion = sys.version_info[0]
@@ -21,7 +25,7 @@ pythonSubVersion = sys.version_info[1]
 class Viewer3D(object):  
     @_tools.KwargChecker(include=[_tools.MPLKwargs])
     def __init__(self,Data,bins,axis=2, log=False ,ax = None, grid = False, adjustable=True, outputFunction=print, 
-                 cmap=None, CurratAxeBraggList=None,Ei=None,EfLimits=None,**kwargs):#pragma: no cover
+                 cmap=None, CurratAxeBraggList=None,Ei=None,EfLimits=None, dataset = None, cut1DFunctionRectangle=None, cut1DFunctionCircle = None, **kwargs):#pragma: no cover
         """3 dimensional viewing object generating interactive Matplotlib figure. 
         Keeps track of all the different plotting functions and variables in order to allow the user to change between different slicing modes and to scroll through the data in an interactive way.
 
@@ -50,6 +54,12 @@ class Viewer3D(object):
             
             - EfLimits (float,float): Minimal and maximal Ef for current instrument (default None)
 
+            - dataset (DataSet): Reference to data set used - needed for interactive cutting with (default None)
+
+            - cut1DFunctionRectangle (function): Function to be called when performing an interactive rectangle (default None)
+
+            - cut1DFunctionCircle (function): Function to be called when performing an interactive rectangle (default None)
+
         For an example, see the `quick plotting tutorial <../Tutorials/Quick/QuickView3D.html>`_ under scripting tutorials.
 
         """
@@ -59,6 +69,21 @@ class Viewer3D(object):
         self.Ei = Ei
         self.EfLimits = EfLimits
 
+        self.outputFunction = outputFunction
+
+        # Settings needed for interactive 1D cutting
+        self.new=False
+        self.line = None
+        self.cidmove = None
+        self.drawState = States.INACTIVE
+        self.ds = dataset
+        self.rects = []
+        self.drs = []
+        self.clearBoxes = lambda: clearBoxes(self)
+
+        self.currentData = None
+        
+
         self.plotCurratAxe = False # Set to false but will change to True when correct list is created
         self._CurratAxeBraggList = None
         if len(Data)==4: # If data is provided as I, norm, mon, normcount
@@ -154,11 +179,49 @@ class Viewer3D(object):
         self.figure.subplots_adjust(bottom=0.25)
         self.cmap = cmap # Update to accomedate deprication warning
         self.value = 0
+        self.setAxis(2)
+        # Set up interactive generation of 1DCuts
+
+        def cut1DFunctionRectangleDefault(self,dr):
+            global cut1DHolder
+            parameters = extractCut1DPropertiesRectangle(dr.rect,self.ax.sample)
+            step = self.dQE[self.axis]
+            pos = np.zeros(3,dtype=int)
+            pos[self.axis]=self.Energy_slider.val
+            
+            EMin = self.bins[self.axis][pos[0],pos[1],pos[2]]
+            EMax = EMin+step
+            cut1DHolder.append([self.ds.plotCut1D(**parameters,Emin=EMin,Emax=EMax)])
+
+
+        def cut1DFunctionCircleDefault(self,dr):
+            global cut1DHolder
+            parameters = extractCut1DPropertiesCircle(dr.circ,self.ax.sample)
+            parameters['E1'] = self.ds.energy.min()
+            parameters['E2'] = self.ds.energy.max()
+            step = self.dQE[self.axis]
+            parameters['minPixel'] = step
+                
+            cut1DHolder.append([self.ds.plotCut1DE(**parameters)])
+
+        if cut1DFunctionRectangle is None:
+            self.cut1DFunctionRectangle = lambda dr: cut1DFunctionRectangleDefault(self,dr)
+        else:
+            self.cut1DFunctionRectangle = lambda dr: cut1DFunctionRectangle(self,dr)
         
+        if cut1DFunctionCircle is None:
+            self.cut1DFunctionCircle = lambda dr: cut1DFunctionCircleDefault(self,dr)
+        else:
+            self.cut1DFunctionCircle = lambda dr: cut1DFunctionCircle(self,dr)
+        
+        Draggables = [DraggableCircle,DraggableRectangle]
+        DraggableFunctions = [self.cut1DFunctionCircle,self.cut1DFunctionRectangle]
+
+        prepareInteractiveCuttingView3D(self,Draggables,DraggableFunctions)
 
         viewAxis = axis
         axis_color='white'
-        self.setAxis(2)
+        
         self.figure.canvas.mpl_connect('key_press_event',lambda event: onkeypress(event, self) )
         self.figure.canvas.mpl_connect('scroll_event',lambda event: onscroll(event, self))
         
@@ -287,7 +350,8 @@ class Viewer3D(object):
             self.caxis = (cmin,cmax)
 
     def setAxis(self,axis):
-        
+        if hasattr(self,'im'): # this function is also called before any plot has been performed
+            self.im.set_array(self.emptyData) # Set data in current im to a fully masked data set
         if axis==2:
             if self.rlu:
                 self.figure.delaxes(self.ax)
@@ -327,6 +391,9 @@ class Viewer3D(object):
         
 
         masked_array = np.ma.array (self.Data, mask=np.isnan(self.Data)).transpose(axes)
+        self.emptyData = masked_array[:,:,0].T.flatten().copy()
+        self.emptyData.mask = np.ones_like(self.emptyData,dtype=bool)
+        self._axesChanged = True
         upperLim = self.Data.shape[axis]-1
         self.label = label
         self.X = X
@@ -340,9 +407,9 @@ class Viewer3D(object):
 
     def calculateValue(self):
         try:
-            val = 0.5*(self.Z[0,0,self.value+1]+self.Z[0,0,self.value])
+            val = 0.5*(self.Z[0,0,int(self.value)+1]+self.Z[0,0,int(self.value)])
         except:
-            val = 0.5*(2*self.Z[0,0,self.value]-self.Z[0,0,self.value-1])
+            val = 0.5*(2*self.Z[0,0,int(self.value)]-self.Z[0,0,int(self.value)-1])
         if hasattr(self,'EnergySliderTransform'):
             val/=self.EnergySliderTransform[self.axis]
         return val
@@ -418,13 +485,18 @@ class Viewer3D(object):
     
     def plot(self):
         self.text.set_text(self.stringValue())
-        self.im.remove()
-        if self.shading=='flat':
-            self.im = self.ax.pcolormesh(self.X[:,:,self.value].T,self.Y[:,:,self.value].T,self.masked_array[:,:,self.value].T,zorder=10,shading=self.shading,edgecolors='face',cmap=self.cmap)
-        elif self.shading=='gouraud': # pragma: no cover
-            XX = 0.5*(self.X[:-1,:-1,self.value]+self.X[1:,1:,self.value]).T
-            YY = 0.5*(self.Y[:-1,:-1,self.value]+self.Y[1:,1:,self.value]).T
-            self.im = self.ax.pcolormesh(XX,YY,self.masked_array[:,:,self.value].T,zorder=10,shading=self.shading,edgecolors='face',cmap=self.cmap) # ,vmin=1e-6,vmax=6e-6
+        try:
+            self.im.set_array(self.emptyData)
+        except TypeError:
+            pass
+        if self._axesChanged:
+            tempData = np.ma.array(self.im.get_array().T)
+            tempData.mask = np.ones_like(tempData,dtype=bool)
+            self.im.set_array(tempData)
+            self._axesChanged = False
+        else:
+            self.im.set_array(self.masked_array[:,:,self.value].T.flatten())
+        
             
         if not self.CurratAxeBraggList is None and not self.Ei is None and self.plotCurratAxe is True and self.rlu is True:
             #if self.axis==2: ### QxQy plane
@@ -492,7 +564,7 @@ def eventdecorator(function,self,event,*args,**kwargs):# pragma: no cover
         except:
             pass
         else:
-            if C != 0:
+            if C != 0 or self.drawState != States.INACTIVE:
                 return
         return function(self,event.xdata,event.ydata,*args,**kwargs)
 
@@ -534,22 +606,27 @@ def onclick(self,x,y,returnText=False, outputFunction=print): # pragma: no cover
     if returnText:
         return printString
     else:
-        outputFunction(printString)
+        if hasattr(self.ax,'suppressPrint'):
+            if not self.ax.suppressPrint:
+                outputFunction(printString)
+        else:
+            outputFunction(printString)
         
 def onkeypress(event,self): # pragma: no cover
-    if event.key in ['+','up','right']:
+    if event.key in Viewer3DSettings['upwards']:
         increaseAxis(event,self)
-    elif event.key in ['-','down','left']:
+    elif event.key in Viewer3DSettings['downwards']:
         decreaseAxis(event,self)
-    elif event.key in ['home']:
+    elif event.key in Viewer3DSettings['home']:
         self.Energy_slider.set_val(self.Energy_slider.valmin)
-    elif event.key in ['end']:
+    elif event.key in Viewer3DSettings['end']:
         self.Energy_slider.set_val(self.Energy_slider.valmax)
 
-    elif event.key in ['0']:
+    elif event.key in Viewer3DSettings['QxE']:
         if self.axis!=0:
             reloadslider(self,0)
-            #del self.im
+            del self.im
+            #self.currentData = None
             if self.shading=='flat':
                 self.im = self.ax.pcolormesh(self.X[:,:,0].T,self.Y[:,:,0].T,self.masked_array[:,:,self.value].T,zorder=10,shading=self.shading,cmap=self.cmap)
             elif self.shading=='gouraud':
@@ -561,10 +638,11 @@ def onkeypress(event,self): # pragma: no cover
             self.plot()
             self.ax.set_xlim([np.min(self.X),np.max(self.X)])
             self.ax.set_ylim([np.min(self.Y),np.max(self.Y)])
-    elif event.key in ['1']:
+    elif event.key in Viewer3DSettings['QyE']:
         if self.axis!=1:
             reloadslider(self,1)
-            #del self.im
+            del self.im
+            #self.currentData = None
             if self.shading=='flat':
                 self.im = self.ax.pcolormesh(self.X[:,:,0].T,self.Y[:,:,0].T,self.masked_array[:,:,self.value].T,zorder=10,shading=self.shading,cmap=self.cmap)
             elif self.shading=='gouraud':
@@ -576,10 +654,11 @@ def onkeypress(event,self): # pragma: no cover
             self.plot()
             self.ax.set_xlim([np.min(self.X),np.max(self.X)])
             self.ax.set_ylim([np.min(self.Y),np.max(self.Y)])
-    elif event.key in ['2']:
+    elif event.key in Viewer3DSettings['QxQy']:
         if self.axis!=2:
             reloadslider(self,2)
-            #del self.im
+            del self.im
+            #self.currentData = None
             if self.shading=='flat':
                 self.im = self.ax.pcolormesh(self.X[:,:,0].T,self.Y[:,:,0].T,self.masked_array[:,:,self.value].T,zorder=10,shading=self.shading,cmap=self.cmap)
             elif self.shading=='gouraud':
@@ -599,7 +678,7 @@ def reloadslider(self,axis): # pragma: no cover
     self.setAxis(axis)
     self.Energy_slider.set_val(0)
     self.Energy_slider.label.remove()
-    self.Energy_slider.disconnect(self.Energy_slider.cids[0])
+    self.Energy_slider.disconnect_events()#(self.Energy_slider.cids[0])
     self.Energy_slider.vline.set_visible(False)
     
     del self.Energy_slider
@@ -613,9 +692,9 @@ def reloadslider(self,axis): # pragma: no cover
     
         
 def onscroll(event,self): # pragma: no cover
-    if(event.button=='up'):
+    if(event.button in Viewer3DSettings['upwardsScroll']):
         increaseAxis(event,self)
-    elif event.button=='down':
+    elif event.button in Viewer3DSettings['downwardsScroll']:
         decreaseAxis(event,self)
 
 
@@ -746,4 +825,16 @@ def addColorbarSliders(self,c_min,c_max,c_minval,c_maxval,ax_cmin,ax_cmax,log=Tr
             
         fig.canvas.draw()
     
-    fig.savefig = savefig
\ No newline at end of file
+    fig.savefig = savefig
+
+def cut1DFunctionDefault(self,dr):
+    global cut1DHolder
+    parameters = extractCut1DProperties(dr.rect,self.ax.sample)
+    step = self.dQE[self.axis]
+    pos = np.zeros(3,dtype=int)
+    pos[self.axis]=self.Energy_slider.val
+    
+    EMin = self.bins[self.axis][pos[0],pos[1],pos[2]]
+    EMax = EMin+step
+    cut1DHolder.append([self.ds.plotCut1D(**parameters,Emin=EMin,Emax=EMax)])
+
